ADDRESSES = include("addresses.lua");

include("player.lua");
include("state_idle.lua");
include("state_moveplayer.lua");
include("functions.lua");

local REDRAW_SECS = 1; -- Time between screen updates

local HEAD_PAIR = 1;
local HP_GOOD_PAIR = 2;
local HP_WARN_PAIR = 3;
local HP_BAD_PAIR = 4;
local ERR_PAIR = 5;

local HP_STYLE;
local HP_GOOD_STYLE;
local HP_WARN_STYLE;
local HP_BAD_STYLE;
local ERR_STYLE;

local startedTime = time.getNow();

function pushState(ns)
	ns.prevState = state;
	state = ns;
end

function popState()
	if( state ) then
		state = state.prevState;
	else
		state = nil;
	end
end

function clearPrevState()
	state.prevState = nil;
end

local lastRedraw = startedTime;
function redrawHud()
	local col2X = 25;
	local col3X = 50;
	ncurses.clear(headWin);

	-- Player name
	ncurses.attributeOn(headWin, A_BOLD);
	ncurses.print(headWin, sprintf("%-8s", "Player:"));
	ncurses.attributeOff(headWin, A_BOLD);
	if( player ) then
		ncurses.print(headWin, player.name);
	end

	-- Player class
	ncurses.move(headWin, 1, 0);
	ncurses.attributeOn(headWin, A_BOLD);
	ncurses.print(headWin, sprintf("%-8s", "Class:"));
	ncurses.attributeOff(headWin, A_BOLD);
	if( player ) then
		ncurses.print(headWin, player.classname);
	end

	-- Display state info
	ncurses.move(headWin, 0, col2X);
	ncurses.attributeOn(headWin, A_BOLD);
	ncurses.print(headWin, sprintf("%-7s", "State:"));
	ncurses.attributeOff(headWin, A_BOLD);
	if( state ) then
		ncurses.print(headWin, state:getName());
	else
		ncurses.print(headWin, "None");
	end

	-- Prep HP bar
	local HP_bar;
	local HP_per;
	local HP_style = 0;
	if( player ) then
		HP_per = player.hp / player.maxhp * 100;
	else
		HP_per = 100;
	end

	if( HP_per < 40 ) then
		HP_style = HP_BAD_STYLE;
	elseif( HP_per < 75 ) then
		HP_style = HP_WARN_STYLE
	else
		HP_style = HP_GOOD_STYLE;
	end

	-- Print HP bar
	ncurses.move(headWin, 1, col2X);
	ncurses.attributeOn(headWin, A_BOLD);
	ncurses.print(headWin, sprintf("%-7s", "HP: "));
	ncurses.attributeOff(headWin, A_BOLD);

	ncurses.setAttribute(headWin, HP_style);
	if( player ) then
		ncurses.print(headWin, sprintf("[%5d/%-5d]", player.hp, player.maxhp));
	else
		ncurses.print(headWin, sprintf("[%5d/%-5d]", 100, 100));
	end
	ncurses.setAttribute(headWin, 0); -- Reset to none

	-- Display run time
	ncurses.move(headWin, 0, col3X);
	ncurses.attributeOn(headWin, A_BOLD);
	ncurses.print(headWin, sprintf("%-12s", "Run time:"));
	ncurses.attributeOff(headWin, A_BOLD);
	ncurses.print(headWin, timeFormat(timer.deltaTime(timer.getNow(), startedTime)));

	-- Display position
	if( player ) then
		ncurses.move(headWin, 1, col3X);
		ncurses.attributeOn(headWin, A_BOLD);
		ncurses.print(headWin, "X: ");
		ncurses.attributeOff(headWin, A_BOLD);
		ncurses.print(headWin, sprintf("%0.1f", player.x));
		ncurses.attributeOn(headWin, A_BOLD);
		ncurses.print(headWin, ", Z: ");
		ncurses.attributeOff(headWin, A_BOLD);
		ncurses.print(headWin, sprintf("%0.1f", player.z));
	end

	-- Finally refresh it
	ncurses.refresh(headWin);

	-- Update redraw time
	lastRedraw = timer.getNow();
end

function macro.init()
	-- Set up Ncurses
	ncurses.init();
	local winMaxY, winMaxX = ncurses.getWindowSize(STDSCR);

	-- Set up our styles
	ncurses.setPair(HEAD_PAIR, ncurses.WHITE, ncurses.BLUE);
	ncurses.setPair(HP_GOOD_PAIR, ncurses.WHITE, ncurses.GREEN);
	ncurses.setPair(HP_WARN_PAIR, ncurses.WHITE, ncurses.YELLOW);
	ncurses.setPair(HP_BAD_PAIR, ncurses.WHITE, ncurses.RED);
	ncurses.setPair(ERR_PAIR, ncurses.RED, ncurses.BLACK);

	HEAD_STYLE = ncurses.getPair(HEAD_PAIR);
	HP_GOOD_STYLE = bit32.bor(ncurses.BOLD, ncurses.getPair(HP_GOOD_PAIR));
	HP_WARN_STYLE = bit32.bor(ncurses.BOLD, ncurses.getPair(HP_WARN_PAIR));
	HP_BAD_STYLE =  bit32.bor(ncurses.BOLD, ncurses.getPair(HP_BAD_PAIR));
	SPLIT_STYLE = ncurses.REVERSE;
	ERR_STYLE = bit32.bor(ncurses.BOLD, ncurses.getPair(ERR_PAIR));

	-- Create windows
	headWin = ncurses.createWindow(0, 0, 2, winMaxX);
	hSplitWin = ncurses.createWindow(2, 0, 1, winMaxX);
	vSplitWin = ncurses.createWindow(3, 60, winMaxY-3, 1);
	logWin = ncurses.createWindow(3, 0, winMaxY-3, 59);
	otherWin = ncurses.createWindow(3, 61, winMaxY-3, winMaxX-61);

	-- Init head
	ncurses.setBackground(headWin, HEAD_STYLE);

	-- Init split(s)
	ncurses.setBackground(hSplitWin, SPLIT_STYLE);
	ncurses.move(hSplitWin, 0, 0);
	ncurses.print(hSplitWin, "[Q]uit  [S]ettings");
	ncurses.move(hSplitWin, 0, 61);
	ncurses.print(hSplitWin, "Something Else");
	ncurses.refresh(hSplitWin);

	ncurses.setBackground(vSplitWin, ncurses.ALTCHARSET);
	local str = "";
	for i = 1, ncurses.getWindowSize(vSplitWin) do
		str = str .. string.char(120);
	end
	ncurses.print(vSplitWin, str);
	ncurses.refresh(vSplitWin);

	-- Init log
	ncurses.scrollok(logWin, true);

	-- Init that other window that we don't know what to do with
	ncurses.print(otherWin, "Not sure what to\nput here...");
	ncurses.refresh(otherWin);

	-- Now do real setup
	player = Player();
	player:init();
	--state = Moveplayer(-574.5, -5906.7);
	pushState(Moveplayer(-574.5, -5906.7));
	pushState(Moveplayer(-391.0, -6193.5));
	pushState(Moveplayer(-302.5, -6371.0));

	-- NOW, finally, redraw the HUD
	redrawHud();
end

local running = true;
function macro.main(dt)
	-- Update memory
	player:update();

	-- Do logic
	if( state ) then -- We have a job to do
		if( state:update(dt) == false ) then
			popState();
		end
	end

	if( state == nil ) then -- Get a new job!
		state = Idle(math.random(2, 10));
	end

	-- Redraw (if needed)
	if( timer.deltaTime(timer.getNow(), lastRedraw) > REDRAW_SECS ) then
		redrawHud();
	end

	-- Continue
	return running;
end

function macro.event(e, data1, data2, data3)
	if( e == "keypressed" ) then
		macro.keypressed(data1, data2);
	end

	if( e == "error" or e == "warning" ) then
		print("ERR/Warn:", data1);
		ncurses.setAttribute(logWin, ERR_STYLE);
		ncurses.print(logWin, sprintf("%s\n", data1));
		ncurses.setAttribute(logWin, 0);
		ncurses.refresh(logWin);
	end
end

function macro.keypressed(vk, toggle)
	if( macro.getFocusHwnd() == macro.getAppHwnd() ) then
		-- Local input only
		if( vk == key.VK_Q ) then
			print("Quit button pressed.");
			running = false;
		end

		if( vk == key.VK_SPACE ) then
			player.hp = math.random(10, 100);
		end
	end
end